
//  File:     mp3_main.cpp
//  Project:  MP3's --- Week 6 HW from Spring 2002
//  Author:   Chuck Stewart
//
//  Purpose:  This is the solution to HW 6 from CS II from the spring
//    semester 2002.   It demonstrates extensive use of maps.  
//
//  Problem:  The program is to build, maintain and answer questions
//    about MP3 files available on various computers.  Each computer
//    is identified by a unique id (a string).  Information about the
//    computer, including connection speed and a list of MP3's stored
//    on the computer are stored in a Computer class object.  The map
//    is therefore an association between a string and a Computer
//    object.  See the accompanying problem description and example
//    input and output for details of the assignment.


#include <algorithm>
#include <iostream>
#include <list>
#include <map>
#include <string>

#include <cctype>

using std::cin;
using std::cout;
using std::endl;
using std::find_if;
using std::getline;
using std::list;
using std::make_pair;
using std::map;
using std::pair;

#include "MP3.h"
#include "Computer.h"

//  Since the names of maps tend to be rather long, typedefs are often
//  used to short the descriptions.  To understand this, simply
//  remember that anywhere in the following code, the name MP3_map is
//  replaced by map< string, Computer >.  This type of substitution
//  only applies to types.

typedef map< string, Computer > MP3_map;


//  Prototypes for major functions

void add_computer( MP3_map & mp3_system );
void remove_computer( MP3_map & mp3_system );
void print_mp3s_on_computer( MP3_map  const& mp3_system );
void find_song( MP3_map  const&mp3_system );
void find_songs_by_artist( MP3_map  const& mp3_system );


// Prototypes for utility functions

bool all_blanks( string const& line );
string trim_line( string const& line );


//  The main function defines the map
int
main()
{
  MP3_map mp3_system;
  bool done = false;

  //  The following loop handles one iterative input at a time.  It
  //  checks for illegal input and invalid options.

  do
    {
      int option;
      if ( ! (cin >> option) )
	{
	  cout << "\nILLEGAL INPUT\n" << endl;
	  return 1;
	}

      switch ( option )
	{
	case 0: 
	  add_computer( mp3_system );
	  break;
	case 1:
	  remove_computer( mp3_system );
	  break;
	case 2:
	  print_mp3s_on_computer( mp3_system );
	  break;
	case 3:
	  find_song( mp3_system );
	  break;
	case 4:
	  find_songs_by_artist( mp3_system );
	  break;
	case 5:
	  done = true;
	  break;
	default:
	  cout << "Error: that is not a valid command" << endl;;
	  break;
	}
    }
  while ( !done );
}


// Add a computer to the map of MP3's

void add_computer( MP3_map  & mp3_system )
{
  //  Read id
  string computer_id;
  double speed;
  cin >> computer_id >> speed;

  //  Create a new computer.
  Computer new_computer( speed );

  //  Remove the rest of the input line.
  string line;
  getline( cin, line );

  //  Get all of the information about the computer's MP3's
  do
    {
      getline( cin, line );
      if ( all_blanks( line ) ) break;
	
      string artist = trim_line( line );
      getline( cin, line );
      string title = trim_line( line );
      double file_size;
      cin >> file_size;
      getline( cin, line );
      
      new_computer . add_mp3( artist, title, file_size );
    }
  while ( true );

  //  Try to add the computer using the map insert function.

  pair< MP3_map::iterator, bool > result_pair
    = mp3_system . insert( make_pair( computer_id, new_computer ) );

  //  Output a message based on the result.

  if ( result_pair . second )
    {
      cout << "Computer " << computer_id << " added." << endl;
    }
  else
    {
      cout << "Computer " << computer_id << " is already in the system." << endl;
    }
}


//  Remove the computer from the system.

void remove_computer( MP3_map  & mp3_system )
{
  //  Read id

  string computer_id;
  cin >> computer_id;

  //  Remove the rest of the input line.

  string line;
  getline( cin, line );

  //  Erase the computer from the system.  Print a message appropriate
  //  to the result.

  if ( mp3_system . erase( computer_id ) )
    {
      cout << "Computer " << computer_id << " removed from system." << endl;
    }
  else
    {
      cout << "Computer " << computer_id << " was not in the system." << endl;
    }
}


//  Print the MP3's for a given computer.

void print_mp3s_on_computer( MP3_map  const& mp3_system )
{
  //  Read id

  string computer_id;
  cin >> computer_id;

  //  Remove the rest of the input line.

  string line;
  getline( cin, line );

  //  Find the computer

  MP3_map :: const_iterator c_itr = mp3_system.find( computer_id );

  if ( c_itr == mp3_system.end() )
    {
      cout << "\nComputer " << computer_id << " is not in the system." << endl;
    }
  else
    {
      cout << "\nSongs found on " << computer_id << "...\n";
      c_itr -> second . print_mp3s();
    }
}


//  Find all MP3s for a given song.  Output information about file
//  size and connection speed.

void find_song( MP3_map  const& mp3_system )
{
  //  Get rest of input line continaing the request (3 in this case)

  string line;
  getline( cin, line );

  //  Get the artist and title from the input.  Trim off leading and
  //  trailing white space from the input lines.

  getline( cin, line );
  string artist = trim_line( line );
  getline( cin, line );
  string title = trim_line( line );
    
  cout << "\nMatches found for \"" << title << "\" by " << artist << "..." << endl;
  int found_count = 0;;
  
  //  Iterate through the map, looking for the song.

  for ( MP3_map :: const_iterator c_itr = mp3_system.begin();
	c_itr != mp3_system.end(); ++ c_itr )
    {
      MP3 request;
      if ( c_itr -> second . has_song( artist, title, request ) )
	{
	  cout << "COMPUTER: " << c_itr -> first 
	       << ", SIZE: " << request . get_size() 
	       << ", ACCESS: " << request . get_access_time() << endl;
	  found_count ++ ;
	}
    }

  cout << found_count << " matches found.\n";
}


//  Find and output information about all songs by a given artist.

void find_songs_by_artist( MP3_map  const& mp3_system )
{
  //  Get rest of input line containing the request (4 in this case)

  string line;
  getline( cin, line );

  //  Get the artist.  Trim off leading and trailing white space from
  //  the input line.

  getline( cin, line );
  string artist = trim_line( line );

  cout << "\nMatches found for all songs by " << artist << "..." << endl;
  int found_count = 0;

  //  Check each line of input.

  for ( MP3_map :: const_iterator c_itr = mp3_system.begin();
	c_itr != mp3_system.end(); ++ c_itr )
    {
      list<MP3> songs_by_artist = c_itr -> second . songs_by_artist( artist );
      if ( ! songs_by_artist.empty() )
	{
	  found_count += songs_by_artist.size();
	  
	  for ( list<MP3>::iterator p = songs_by_artist.begin();
		p != songs_by_artist.end(); ++ p )
	    {
	      cout << "COMPUTER: " << c_itr -> first 
		   << ", TITLE: " << p -> get_title()
		   << ", SIZE: " << p -> get_size()
		   << ", ACCESS: " << p -> get_access_time() << "\n";
	    }
	}
    }

  cout << found_count << " matches found." << endl;
}


bool nonspace( char c )
{
  return !isspace(c);
}


//  Utility function to determine if a line contains all blanks.

bool all_blanks( string const& line )
{
  return find_if( line.begin(), line.end(), nonspace ) == line.end();
}


//  Trim leading and trailing whitespace from an input line, returing
//  the resulting string.

string trim_line( string const& line )
{
  int first, last;

  for ( first = 0; isspace( line[first] ) && first < line.size(); ++ first )
    ;

  for ( last = line.size()-1; isspace( line[last] ) && last >= 0; -- last )
    ;

  return line.substr( first, last-first+1 );
}


